//+------------------------------------------------------------------+
//|                                          consecutive candles.mq4 |
//+------------------------------------------------------------------+
#property link      "https://www.forexfactory.com/icerice"
#property version   "1.04"
#property strict
#property indicator_chart_window
#property indicator_buffers 11
#property indicator_plots   11
//--- plot BuySignal
#property indicator_label1  "BuySignal"
#property indicator_type1   DRAW_ARROW
#property indicator_color1  clrLimeGreen
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot SellSignal
#property indicator_label2  "SellSignal"
#property indicator_type2   DRAW_ARROW
#property indicator_color2  clrRed
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- plot BuyDot
#property indicator_label3  "BuyDotSignal"
#property indicator_type3   DRAW_ARROW
#property indicator_color3  clrLimeGreen
#property indicator_style3  STYLE_SOLID
#property indicator_width3  1
//--- plot SellDot
#property indicator_label4  "SellDotSignal"
#property indicator_type4   DRAW_ARROW
#property indicator_color4  clrRed
#property indicator_style4  STYLE_SOLID
#property indicator_width4  1


//--- input parameters
input bool     VortexFilter=true;      // Vortex Filter
input int      VortexLength=14;        // Vortex Length
input bool     ShowAlert=true;
input bool     ShowArrows=true;
input bool     ShowDots=true;

//--- indicator buffers
double         BuySignalBuffer[];
double         SellSignalBuffer[];
double         BuyDotBuffer[];
double         SellDotBuffer[];

double         PlusVI[];            //VI+ : + Vortex Indicator buffer
double         MinusVI[];           //VI- : - Vortex Indicator buffer
double         PlusVM[];            //VM+ : + Vortex Movement buffer
double         MinusVM[];           //VM- : - Vorext Movement buffer
double         SumPlusVM[];         //Sum of VI_Length PlusVM values
double         SumMinusVM[];        //Sum of VI_Length MinusVM values
double         SumTR[];             //True Range buffer

datetime LastAlertTime;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,BuySignalBuffer);
   SetIndexBuffer(1,SellSignalBuffer);
   SetIndexBuffer(2,BuyDotBuffer);
   SetIndexBuffer(3,SellDotBuffer);
   
   SetIndexBuffer(4,PlusVI, INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,MinusVI, INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,PlusVM, INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,MinusVM, INDICATOR_CALCULATIONS);
   SetIndexBuffer(8,SumPlusVM, INDICATOR_CALCULATIONS);
   SetIndexBuffer(9,SumMinusVM, INDICATOR_CALCULATIONS);
   SetIndexBuffer(10,SumTR, INDICATOR_CALCULATIONS);
   
//--- setting a code from the Wingdings charset as the property of PLOT_ARROW
   SetIndexArrow(0,233);
   SetIndexArrow(1,234);
   SetIndexArrow(2,159);
   SetIndexArrow(3,159);
   
   if (!ShowArrows)
   {
      SetIndexStyle(0, DRAW_NONE);
      SetIndexStyle(1, DRAW_NONE);
   }
   
   if (!ShowDots)
   {
      SetIndexStyle(2, DRAW_NONE);
      SetIndexStyle(3, DRAW_NONE);
   }
   
   SetIndexStyle(4, DRAW_NONE);
   SetIndexStyle(5, DRAW_NONE);
   SetIndexStyle(6, DRAW_NONE);
   SetIndexStyle(7, DRAW_NONE);
   SetIndexStyle(8, DRAW_NONE);
   SetIndexStyle(9, DRAW_NONE);
   SetIndexStyle(10, DRAW_NONE);
   
   SetIndexEmptyValue(0, EMPTY_VALUE);
   SetIndexEmptyValue(1, EMPTY_VALUE);
   SetIndexEmptyValue(2, EMPTY_VALUE);
   SetIndexEmptyValue(3, EMPTY_VALUE);
   SetIndexEmptyValue(4, EMPTY_VALUE);
   SetIndexEmptyValue(5, EMPTY_VALUE);
   SetIndexEmptyValue(6, EMPTY_VALUE);
   SetIndexEmptyValue(7, EMPTY_VALUE);
   SetIndexEmptyValue(8, EMPTY_VALUE);
   SetIndexEmptyValue(9, EMPTY_VALUE);
   SetIndexEmptyValue(10, EMPTY_VALUE);
      
   ArraySetAsSeries(BuySignalBuffer, true);
   ArraySetAsSeries(SellSignalBuffer, true);
   ArraySetAsSeries(BuyDotBuffer, true);
   ArraySetAsSeries(SellDotBuffer, true);   
   ArraySetAsSeries(PlusVI, true);
   ArraySetAsSeries(MinusVI, true);
   ArraySetAsSeries(PlusVM, true);
   ArraySetAsSeries(MinusVM, true);
   ArraySetAsSeries(SumPlusVM, true);
   ArraySetAsSeries(SumMinusVM, true);
   ArraySetAsSeries(SumTR, true);
   
   ArrayInitialize(BuySignalBuffer, EMPTY_VALUE);
   ArrayInitialize(SellSignalBuffer, EMPTY_VALUE);
   ArrayInitialize(BuyDotBuffer, EMPTY_VALUE);
   ArrayInitialize(SellDotBuffer, EMPTY_VALUE);   
   ArrayInitialize(PlusVI, EMPTY_VALUE);
   ArrayInitialize(MinusVI, EMPTY_VALUE);
   ArrayInitialize(PlusVM, EMPTY_VALUE);
   ArrayInitialize(MinusVM, EMPTY_VALUE);
   ArrayInitialize(SumPlusVM, EMPTY_VALUE);
   ArrayInitialize(SumMinusVM, EMPTY_VALUE);
   ArrayInitialize(SumTR, EMPTY_VALUE);
         
   LastAlertTime = TimeCurrent()-PeriodSeconds();

//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int limit = rates_total;
   if (prev_calculated > 0) limit = rates_total-prev_calculated;
   if (limit >= rates_total - 2) limit = rates_total - 3;
   if (VortexFilter && rates_total - limit < VortexLength) limit = rates_total - VortexLength - 1;
   
   if (VortexFilter)
   {
      //---- Clear caching buffers
      for(int i = 0; i < limit; i++)
      {
         SumPlusVM[i] = 0;
         SumMinusVM[i] = 0;
         SumTR[i]= 0;  
      }
      //---- Store the values of PlusVM and MinusVM
      for(int i = 0; i < limit; i++)
      {
         //PlusVM = |Today's High - Yesterday's Low|
         PlusVM[i] = MathAbs(High[i] - Low[i + 1]);
         //MinusVM = |Today's Low - Yesterday's High|
         MinusVM[i] = MathAbs(Low[i] - High[i +1]);  
       }
      //---- Sum VI_Length values of PlusVM, MinusVM and the True Range
      for(int i = 0; i < limit; i++)
      {
         for(int j = 0; j <= VortexLength - 1; j++)
         {
            SumPlusVM[i] += PlusVM[i + j];
            SumMinusVM[i] += MinusVM[i + j];
            SumTR[i] += iATR(NULL,0,1,i + j); //Sum VI_Length values of the True Range by using a 1-period ATR
         }
      }   
      //---- Draw the indicator
      for(int i = 0; i < limit; i++)
      {
         PlusVI[i] = SumPlusVM[i] / SumTR[i];
         MinusVI[i] = SumMinusVM[i] / SumTR[i];
      }   
   }

   for (int i=limit; i>0; i--)
   {
      BuySignalBuffer[i] = EMPTY_VALUE;
      SellSignalBuffer[i] = EMPTY_VALUE;
      BuyDotBuffer[i] = EMPTY_VALUE;
      SellDotBuffer[i] = EMPTY_VALUE;
      int lastSignalType = -1;
      double avRange = 0;
            
      if (High[i] > High[i+1] && Close[i] > High[i+1])
      {
         if (!VortexFilter || PlusVI[i] > MinusVI[i])
         {
            avRange = AvRange(i, high, low, rates_total);
            BuyDotBuffer[i] = Low[i] - avRange / 1.5;
            lastSignalType = GetLastSignalType(i);
            if (lastSignalType != OP_BUY) BuySignalBuffer[i] = Low[i] - avRange;
         }
      }
      if (Low[i] < Low[i+1] && Close[i] < Low[i+1])
      {
         if (!VortexFilter || PlusVI[i] < MinusVI[i])
         {
            avRange = AvRange(i, high, low, rates_total);
            SellDotBuffer[i] = High[i] + avRange / 1.5;
            lastSignalType = GetLastSignalType(i);
            if (lastSignalType != OP_SELL) SellSignalBuffer[i] = High[i] + avRange;
         }
      }
   }
   
   if (ShowAlert && Time[0] > LastAlertTime)
   {  
      string msg = "";
      if (NormalizeDouble(BuySignalBuffer[1],Digits) != EMPTY_VALUE)
      {
         msg = WindowExpertName()+" BUY signal for "+_Symbol+" on "+EnumToString((ENUM_TIMEFRAMES)_Period)+" at "+TimeToStr(Time[1]);
         Alert(msg);
         LastAlertTime = Time[0];
      }
      else if (NormalizeDouble(SellSignalBuffer[1],Digits) != EMPTY_VALUE)
      {
         msg = WindowExpertName()+" SELL signal for "+_Symbol+" on "+EnumToString((ENUM_TIMEFRAMES)_Period)+" at "+TimeToStr(Time[1]);
         Alert(msg);
         LastAlertTime = Time[0];
      }         
   }
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
  
double AvRange(int pos, double const &high[], double const &low[], double rates_total)
{
   double sum=0;   
   int count = 4;
   int i=pos;
   
   while (i<rates_total && count > 0)
   {
      sum += high[i]-low[i];
      count--;
      i++;
   }
   return (sum/8);
}

int GetLastSignalType(int startIndex)
{
   int signalType = -1;
   int i = startIndex + 1;
   while (i < Bars && signalType == -1)
   {
      if (BuyDotBuffer[i]!= EMPTY_VALUE)
      {
         signalType = OP_BUY;
      }
      else if (SellDotBuffer[i] != EMPTY_VALUE)
      {
         signalType = OP_SELL;
      }
      i++;
   }
   return (signalType);
}
//+------------------------------------------------------------------+
